@@ -1,7 +1,7 @@
 // Copyright (C) 2024-2025, Benjamin Drung <bdrung@posteo.de>
 // SPDX-License-Identifier: ISC
 
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::fs::{
     create_dir, create_dir_all, hard_link, remove_file, set_permissions, symlink_metadata, File,
     OpenOptions,
@@ -17,10 +17,19 @@ use crate::compression::read_magic_header;
 use crate::filetype::*;
 use crate::header::Header;
 use crate::libc::{mknod, set_modified};
+use crate::logger::Logger;
 use crate::ranges::Ranges;
 use crate::seek_forward::SeekForward;
-use crate::{filename_matches, seek_to_cpio_end, SeenFiles, LOG_LEVEL_DEBUG, LOG_LEVEL_INFO};
+use crate::{filename_matches, seek_to_cpio_end, TRAILER_FILENAME};
 
+// TODO: Document hardlink structure
+pub(crate) type SeenFiles = HashMap<u128, String>;
+
+/// Options for extracting cpio archives.
+///
+/// **Warning**: This struct was designed for the `extract_cpio_archive` function.
+/// The API can change between releases and no stability promises are given.
+/// Please get in contact to support your use case and make the API for this function stable.
 #[derive(Clone, Debug, PartialEq)]
 pub struct ExtractOptions {
     make_directories: bool,
@@ -31,6 +40,11 @@ pub struct ExtractOptions {
 }
 
 impl ExtractOptions {
+    /// Create a new extract options structure.
+    ///
+    /// **Warning**: This function was designed for the `3cpio` command-line application.
+    /// The API can change between releases and no stability promises are given.
+    /// Please get in contact to support your use case and make the API for this function stable.
     pub fn new(
         make_directories: bool,
         parts: Option<Ranges>,
@@ -67,11 +81,9 @@ impl Extractor {
         }
     }
 
-    fn set_modified_times(&self, log_level: u32) -> Result<()> {
+    fn set_modified_times<W: Write>(&self, logger: &mut Logger<W>) -> Result<()> {
         for (path, mtime) in self.mtimes.iter().rev() {
-            if log_level >= LOG_LEVEL_DEBUG {
-                writeln!(std::io::stderr(), "set mtime {mtime} for '{path}'")?;
-            };
+            debug!(logger, "set mtime {mtime} for '{path}'")?;
             set_modified(path, *mtime)?;
         }
         Ok(())
@@ -129,19 +141,24 @@ fn create_dir_ignore_existing<P: AsRef<std::path::Path>>(path: P) -> Result<()>
     Ok(())
 }
 
-pub fn extract_cpio_archive<W: Write>(
+/// Extract cpio archives.
+///
+/// **Warning**: This function was designed for the `3cpio` command-line application.
+/// The API can change between releases and no stability promises are given.
+/// Please get in contact to support your use case and make the API for this function stable.
+pub fn extract_cpio_archive<W: Write, LW: Write>(
     mut archive: File,
     mut out: Option<&mut W>,
     options: &ExtractOptions,
-    log_level: u32,
+    logger: &mut Logger<LW>,
 ) -> Result<()> {
     let mut count = 0;
     let base_dir = std::env::current_dir()?;
     loop {
         count += 1;
-        let compression = match read_magic_header(&mut archive) {
+        let compression = match read_magic_header(&mut archive)? {
             None => return Ok(()),
-            Some(x) => x?,
+            Some(x) => x,
         };
         if options.parts.as_ref().is_some_and(|f| !f.contains(&count)) {
             if compression.is_uncompressed() && options.parts.as_ref().unwrap().has_more(&count) {
@@ -157,10 +174,10 @@ pub fn extract_cpio_archive<W: Write>(
             std::env::set_current_dir(&dir)?;
         }
         if compression.is_uncompressed() {
-            read_cpio_and_extract(&mut archive, &dir, &mut out, options, log_level)?;
+            read_cpio_and_extract(&mut archive, &dir, &mut out, options, logger)?;
         } else {
             let mut decompressed = compression.decompress(archive)?;
-            read_cpio_and_extract(&mut decompressed, &dir, &mut out, options, log_level)?;
+            read_cpio_and_extract(&mut decompressed, &dir, &mut out, options, logger)?;
             break;
         }
     }
@@ -171,19 +188,19 @@ fn from_mtime(mtime: u32) -> SystemTime {
     std::time::UNIX_EPOCH + std::time::Duration::from_secs(mtime.into())
 }
 
-fn read_cpio_and_extract<R: Read + SeekForward, W: Write>(
+fn read_cpio_and_extract<R: Read + SeekForward, W: Write, LW: Write>(
     archive: &mut R,
     base_dir: &PathBuf,
     out: &mut Option<W>,
     options: &ExtractOptions,
-    log_level: u32,
+    logger: &mut Logger<LW>,
 ) -> Result<()> {
     let mut extractor = Extractor::new();
     let mut previous_checked_dir = PathBuf::new();
     loop {
         let header = match Header::read(archive) {
             Ok(header) => {
-                if header.filename == "TRAILER!!!" {
+                if header.filename == TRAILER_FILENAME {
                     break;
                 } else {
                     header
@@ -192,18 +209,14 @@ fn read_cpio_and_extract<R: Read + SeekForward, W: Write>(
             Err(e) => return Err(e),
         };
 
-        if log_level >= LOG_LEVEL_DEBUG {
-            writeln!(std::io::stderr(), "{header:?}")?;
-        }
+        debug!(logger, "{header:?}")?;
 
         if !options.patterns.is_empty() && !filename_matches(&header.filename, &options.patterns) {
             header.skip_file_content(archive)?;
             continue;
         }
 
-        if log_level >= LOG_LEVEL_INFO {
-            writeln!(std::io::stderr(), "{}", header.filename)?;
-        }
+        info!(logger, "{}", header.filename)?;
 
         if out.is_none() && !header.is_root_directory() {
             // TODO: use dirfd once stable: https://github.com/rust-lang/rust/issues/120426
@@ -251,12 +264,12 @@ fn read_cpio_and_extract<R: Read + SeekForward, W: Write>(
         } else {
             match header.mode & MODE_FILETYPE_MASK {
                 FILETYPE_CHARACTER_DEVICE => {
-                    write_character_device(&header, options.preserve_permissions, log_level)?
+                    write_character_device(&header, options.preserve_permissions, logger)?
                 }
                 FILETYPE_DIRECTORY => write_directory(
                     &header,
                     options.preserve_permissions,
-                    log_level,
+                    logger,
                     &mut extractor.mtimes,
                 )?,
                 FILETYPE_REGULAR_FILE => write_file(
@@ -264,10 +277,10 @@ fn read_cpio_and_extract<R: Read + SeekForward, W: Write>(
                     &header,
                     options.preserve_permissions,
                     &mut extractor.seen_files,
-                    log_level,
+                    logger,
                 )?,
                 FILETYPE_SYMLINK => {
-                    write_symbolic_link(archive, &header, options.preserve_permissions, log_level)?
+                    write_symbolic_link(archive, &header, options.preserve_permissions, logger)?
                 }
                 FILETYPE_FIFO | FILETYPE_BLOCK_DEVICE | FILETYPE_SOCKET => {
                     unimplemented!(
@@ -287,14 +300,14 @@ fn read_cpio_and_extract<R: Read + SeekForward, W: Write>(
             }
         };
     }
-    extractor.set_modified_times(log_level)?;
+    extractor.set_modified_times(logger)?;
     Ok(())
 }
 
-fn write_character_device(
+fn write_character_device<W: Write>(
     header: &Header,
     preserve_permissions: bool,
-    log_level: u32,
+    logger: &mut Logger<W>,
 ) -> Result<()> {
     if header.filesize != 0 {
         return Err(Error::new(
@@ -305,14 +318,12 @@ fn write_character_device(
             ),
         ));
     };
-    if log_level >= LOG_LEVEL_DEBUG {
-        writeln!(
-            std::io::stderr(),
-            "Creating character device '{}' with mode {:o}",
-            header.filename,
-            header.mode_perm(),
-        )?;
-    };
+    debug!(
+        logger,
+        "Creating character device '{}' with mode {:o}",
+        header.filename,
+        header.mode_perm(),
+    )?;
     if let Err(e) = mknod(&header.filename, header.mode, header.rmajor, header.rminor) {
         match e.kind() {
             ErrorKind::AlreadyExists => {
@@ -332,10 +343,10 @@ fn write_character_device(
     Ok(())
 }
 
-fn write_directory(
+fn write_directory<W: Write>(
     header: &Header,
     preserve_permissions: bool,
-    log_level: u32,
+    logger: &mut Logger<W>,
     mtimes: &mut BTreeMap<String, i64>,
 ) -> Result<()> {
     if header.filesize != 0 {
@@ -347,19 +358,17 @@ fn write_directory(
             ),
         ));
     };
-    if log_level >= LOG_LEVEL_DEBUG {
-        writeln!(
-            std::io::stderr(),
-            "Creating directory '{}' with mode {:o}{}",
-            header.filename,
-            header.mode_perm(),
-            if preserve_permissions {
-                format!(" and owner {}:{}", header.uid, header.gid)
-            } else {
-                String::new()
-            },
-        )?;
-    };
+    debug!(
+        logger,
+        "Creating directory '{}' with mode {:o}{}",
+        header.filename,
+        header.mode_perm(),
+        if preserve_permissions {
+            format!(" and owner {}:{}", header.uid, header.gid)
+        } else {
+            String::new()
+        },
+    )?;
     create_dir_ignore_existing(&header.filename)?;
     if preserve_permissions {
         chown(&header.filename, Some(header.uid), Some(header.gid))?;
@@ -369,30 +378,28 @@ fn write_directory(
     Ok(())
 }
 
-fn write_file<R: Read + SeekForward>(
+fn write_file<R: Read + SeekForward, W: Write>(
     archive: &mut R,
     header: &Header,
     preserve_permissions: bool,
     seen_files: &mut SeenFiles,
-    log_level: u32,
+    logger: &mut Logger<W>,
 ) -> Result<()> {
     let mut file;
     if let Some(target) = header.try_get_hard_link_target(seen_files) {
-        if log_level >= LOG_LEVEL_DEBUG {
-            writeln!(
-                std::io::stderr(),
-                "Creating hard-link '{}' -> '{}' with permission {:o}{} and {} bytes",
-                header.filename,
-                target,
-                header.mode_perm(),
-                if preserve_permissions {
-                    format!(" and owner {}:{}", header.uid, header.gid)
-                } else {
-                    String::new()
-                },
-                header.filesize,
-            )?;
-        };
+        debug!(
+            logger,
+            "Creating hard-link '{}' -> '{}' with permission {:o}{} and {} bytes",
+            header.filename,
+            target,
+            header.mode_perm(),
+            if preserve_permissions {
+                format!(" and owner {}:{}", header.uid, header.gid)
+            } else {
+                String::new()
+            },
+            header.filesize,
+        )?;
         if let Err(e) = hard_link(target, &header.filename) {
             match e.kind() {
                 ErrorKind::AlreadyExists => {
@@ -406,20 +413,18 @@ fn write_file<R: Read + SeekForward>(
         }
         file = OpenOptions::new().write(true).open(&header.filename)?
     } else {
-        if log_level >= LOG_LEVEL_DEBUG {
-            writeln!(
-                std::io::stderr(),
-                "Creating file '{}' with permission {:o}{} and {} bytes",
-                header.filename,
-                header.mode_perm(),
-                if preserve_permissions {
-                    format!(" and owner {}:{}", header.uid, header.gid)
-                } else {
-                    String::new()
-                },
-                header.filesize,
-            )?;
-        };
+        debug!(
+            logger,
+            "Creating file '{}' with permission {:o}{} and {} bytes",
+            header.filename,
+            header.mode_perm(),
+            if preserve_permissions {
+                format!(" and owner {}:{}", header.uid, header.gid)
+            } else {
+                String::new()
+            },
+            header.filesize,
+        )?;
         file = File::create(&header.filename)?
     };
     header.mark_seen(seen_files);
@@ -450,22 +455,20 @@ fn write_file_content<R: Read + SeekForward, W: Write>(
     header.skip_file_content_padding(archive)
 }
 
-fn write_symbolic_link<R: Read + SeekForward>(
+fn write_symbolic_link<R: Read + SeekForward, W: Write>(
     archive: &mut R,
     header: &Header,
     preserve_permissions: bool,
-    log_level: u32,
+    logger: &mut Logger<W>,
 ) -> Result<()> {
     let target = header.read_symlink_target(archive)?;
-    if log_level >= LOG_LEVEL_DEBUG {
-        writeln!(
-            std::io::stderr(),
-            "Creating symlink '{}' -> '{}' with mode {:o}",
-            header.filename,
-            &target,
-            header.mode_perm(),
-        )?;
-    };
+    debug!(
+        logger,
+        "Creating symlink '{}' -> '{}' with mode {:o}",
+        header.filename,
+        &target,
+        header.mode_perm(),
+    )?;
     if let Err(e) = symlink(&target, &header.filename) {
         match e.kind() {
             ErrorKind::AlreadyExists => {
@@ -501,9 +504,9 @@ mod tests {
 
     use super::*;
     use crate::libc::{major, minor};
+    use crate::logger::Level;
     use crate::temp_dir::TempDir;
     use crate::tests::{tests_path, TEST_LOCK};
-    use crate::LOG_LEVEL_WARNING;
 
     fn getgid() -> u32 {
         unsafe { ::libc::getgid() }
@@ -544,11 +547,13 @@ mod tests {
         let tempdir = TempDir::new_and_set_current_dir().unwrap();
         let patterns = vec![Pattern::new("p?th/f*").unwrap()];
         let options = ExtractOptions::new(true, None, patterns, false, None);
+        let mut logger = Logger::new_vec(Level::Info);
 
-        extract_cpio_archive(archive, None::<&mut Stdout>, &options, LOG_LEVEL_WARNING).unwrap();
+        extract_cpio_archive(archive, None::<&mut Stdout>, &options, &mut logger).unwrap();
         assert!(tempdir.path.join("path").is_dir());
         assert!(tempdir.path.join("path/file").exists());
         assert!(!tempdir.path.join("usr").exists());
+        assert_eq!(logger.get_logs(), "path/file\n");
     }
 
     #[test]
@@ -562,8 +567,10 @@ mod tests {
             false,
             None,
         );
-        extract_cpio_archive(archive, Some(&mut output), &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        extract_cpio_archive(archive, Some(&mut output), &options, &mut logger).unwrap();
         assert_eq!(String::from_utf8(output).unwrap(), "content\n");
+        assert_eq!(logger.get_logs(), ".\npath\npath/file\n");
     }
 
     #[test]
@@ -571,11 +578,13 @@ mod tests {
         let archive = File::open(tests_path("bzip2.cpio")).unwrap();
         let mut output = Vec::new();
         let options = ExtractOptions::default();
-        extract_cpio_archive(archive, Some(&mut output), &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        extract_cpio_archive(archive, Some(&mut output), &options, &mut logger).unwrap();
         assert_eq!(
             String::from_utf8(output).unwrap(),
             "content\nThis is a fake busybox binary to simulate a POSIX shell\n"
         );
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -585,9 +594,30 @@ mod tests {
         let tempdir = TempDir::new_and_set_current_dir().unwrap();
         let patterns = vec![Pattern::new("p?th").unwrap()];
         let options = ExtractOptions::new(false, None, patterns, false, None);
-        extract_cpio_archive(archive, None::<&mut Stdout>, &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
+        extract_cpio_archive(archive, None::<&mut Stdout>, &options, &mut logger).unwrap();
         assert!(tempdir.path.join("path").is_dir());
         assert!(!tempdir.path.join("path/file").exists());
+        assert_eq!(
+            logger.get_logs(),
+            "Header { ino: 0, mode: 16893, uid: 0, gid: 0, nlink: 2, mtime: 1713104326, filesize: 0, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \".\" }\n\
+            Header { ino: 1, mode: 16893, uid: 0, gid: 0, nlink: 2, mtime: 1713104326, filesize: 0, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \"path\" }\n\
+            path\n\
+            Creating directory 'path' with mode 775\n\
+            Header { ino: 2, mode: 33204, uid: 0, gid: 0, nlink: 1, mtime: 1713104326, filesize: 8, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \"path/file\" }\n\
+            set mtime 1713104326 for 'path'\n\
+            Header { ino: 0, mode: 16893, uid: 0, gid: 0, nlink: 2, mtime: 1713104326, filesize: 0, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \".\" }\n\
+            Header { ino: 1, mode: 16893, uid: 0, gid: 0, nlink: 2, mtime: 1713104326, filesize: 0, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \"usr\" }\n\
+            Header { ino: 2, mode: 16893, uid: 0, gid: 0, nlink: 2, mtime: 1713104326, filesize: 0, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \"usr/bin\" }\n\
+            Header { ino: 3, mode: 33204, uid: 0, gid: 0, nlink: 1, mtime: 1713104326, filesize: 56, \
+            major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \"usr/bin/sh\" }\n"
+        );
     }
 
     #[test]
@@ -596,11 +626,13 @@ mod tests {
         let patterns: Vec<Pattern> = vec![Pattern::new("*/b?n/sh").unwrap()];
         let mut output = Vec::new();
         let options = ExtractOptions::new(false, None, patterns, false, None);
-        extract_cpio_archive(archive, Some(&mut output), &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        extract_cpio_archive(archive, Some(&mut output), &options, &mut logger).unwrap();
         assert_eq!(
             String::from_utf8(output).unwrap(),
             "This is a fake busybox binary to simulate a POSIX shell\n"
         );
+        assert_eq!(logger.get_logs(), "usr/bin/sh\n");
     }
 
     #[test]
@@ -610,9 +642,11 @@ mod tests {
         let tempdir = TempDir::new_and_set_current_dir().unwrap();
         let patterns = vec![Pattern::new("path").unwrap()];
         let options = ExtractOptions::new(false, None, patterns, false, None);
-        extract_cpio_archive(archive, None::<&mut Stdout>, &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        extract_cpio_archive(archive, None::<&mut Stdout>, &options, &mut logger).unwrap();
         assert!(tempdir.path.join("path").is_dir());
         assert!(!tempdir.path.join("path/file").exists());
+        assert_eq!(logger.get_logs(), "path\n");
     }
 
     #[test]
@@ -621,9 +655,11 @@ mod tests {
         let archive = File::open(tests_path("single.cpio")).unwrap();
         let tempdir = TempDir::new_and_set_current_dir().unwrap();
         let options = ExtractOptions::new(false, None, Vec::new(), false, Some("cpio".into()));
-        extract_cpio_archive(archive, None::<&mut Stdout>, &options, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        extract_cpio_archive(archive, None::<&mut Stdout>, &options, &mut logger).unwrap();
         let path = tempdir.path.join("cpio1/path/file");
         assert!(path.exists());
+        assert_eq!(logger.get_logs(), ".\npath\npath/file\n");
     }
 
     // Test detecting path traversal attacks like CVE-2015-1197
@@ -632,12 +668,13 @@ mod tests {
         let _lock = TEST_LOCK.lock().unwrap();
         let mut archive = File::open(tests_path("path-traversal.cpio")).unwrap();
         let tempdir = TempDir::new_and_set_current_dir().unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
         let got = read_cpio_and_extract(
             &mut archive,
             &tempdir.path,
             &mut None::<Stdout>,
             &ExtractOptions::default(),
-            LOG_LEVEL_WARNING,
+            &mut logger,
         )
         .unwrap_err();
         assert_eq!(got.kind(), ErrorKind::InvalidData);
@@ -645,6 +682,7 @@ mod tests {
             "The parent directory of \"tmp/trav.txt\" (resolved to \"/tmp\") is not within the directory {:#?}.",
             &tempdir.path
         ));
+        assert_eq!(logger.get_logs(), ".\ntmp\ntmp/trav.txt\n");
     }
 
     #[test]
@@ -652,15 +690,17 @@ mod tests {
         let mut archive = File::open(tests_path("path-traversal.cpio")).unwrap();
         let base_dir = std::env::current_dir().unwrap();
         let mut output = Vec::new();
+        let mut logger = Logger::new_vec(Level::Info);
         read_cpio_and_extract(
             &mut archive,
             &base_dir,
             &mut Some(&mut output),
             &ExtractOptions::default(),
-            LOG_LEVEL_WARNING,
+            &mut logger,
         )
         .unwrap();
         assert_eq!(String::from_utf8(output).unwrap(), "TEST Traversal\n");
+        assert_eq!(logger.get_logs(), ".\ntmp\ntmp/trav.txt\n");
     }
 
     #[test]
@@ -674,7 +714,8 @@ mod tests {
         let mut header = Header::new(1, 0o20_644, 0, 0, 0, 1740402179, 0, 0, 0, "./null");
         header.rmajor = 1;
         header.rminor = 3;
-        write_character_device(&header, true, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
+        write_character_device(&header, true, &mut logger).unwrap();
 
         let attr = std::fs::metadata("null").unwrap();
         assert_eq!(attr.len(), header.filesize.into());
@@ -685,6 +726,10 @@ mod tests {
         assert_eq!(attr.gid(), header.gid);
         assert_eq!(major(attr.rdev()), header.rmajor);
         assert_eq!(minor(attr.rdev()), header.rminor);
+        assert_eq!(
+            logger.get_logs(),
+            "Creating character device './null' with mode 644\n"
+        );
         std::fs::remove_file("null").unwrap();
     }
 
@@ -705,13 +750,22 @@ mod tests {
             0,
             "./directory_with_setuid",
         );
-        write_directory(&header, true, LOG_LEVEL_WARNING, &mut mtimes).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
+        write_directory(&header, true, &mut logger, &mut mtimes).unwrap();
 
         let attr = std::fs::metadata("directory_with_setuid").unwrap();
         assert!(attr.is_dir());
         assert_eq!(attr.permissions(), PermissionsExt::from_mode(header.mode));
         assert_eq!(attr.uid(), header.uid);
         assert_eq!(attr.gid(), header.gid);
+        assert_eq!(
+            logger.get_logs(),
+            format!(
+                "Creating directory './directory_with_setuid' with mode 3777 and owner {}:{}\n",
+                getuid(),
+                getgid(),
+            ),
+        );
         std::fs::remove_dir("directory_with_setuid").unwrap();
 
         let mut expected_mtimes: BTreeMap<String, i64> = BTreeMap::new();
@@ -737,12 +791,13 @@ mod tests {
             "./file_with_setuid",
         );
         let cpio = b"!/bin/sh\n\0\0\0";
+        let mut logger = Logger::new_vec(Level::Debug);
         write_file(
             &mut cpio.as_ref(),
             &header,
             true,
             &mut seen_files,
-            LOG_LEVEL_WARNING,
+            &mut logger,
         )
         .unwrap();
 
@@ -753,6 +808,14 @@ mod tests {
         assert_eq!(attr.permissions(), PermissionsExt::from_mode(header.mode));
         assert_eq!(attr.uid(), header.uid);
         assert_eq!(attr.gid(), header.gid);
+        assert_eq!(
+            logger.get_logs(),
+            format!(
+                "Creating file './file_with_setuid' with permission 4755 and owner {}:{} and 9 bytes\n",
+                getuid(),
+                getgid(),
+            ),
+        );
         std::fs::remove_file("file_with_setuid").unwrap();
     }
 
@@ -773,7 +836,8 @@ mod tests {
             "./dead_symlink",
         );
         let cpio = b"/nonexistent";
-        write_symbolic_link(&mut cpio.as_ref(), &header, true, LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        write_symbolic_link(&mut cpio.as_ref(), &header, true, &mut logger).unwrap();
 
         let attr = std::fs::symlink_metadata("dead_symlink").unwrap();
         assert_eq!(attr.len(), header.filesize.into());
@@ -782,6 +846,7 @@ mod tests {
         assert_eq!(attr.permissions(), PermissionsExt::from_mode(header.mode));
         assert_eq!(attr.uid(), header.uid);
         assert_eq!(attr.gid(), header.gid);
+        assert_eq!(logger.get_logs(), "");
         std::fs::remove_file("dead_symlink").unwrap();
     }
 }
