@@ -3,36 +3,39 @@
 
 use std::fs::Permissions;
 use std::io::{Error, ErrorKind, Read, Result, Write};
+use std::num::NonZeroU32;
 use std::os::unix::fs::PermissionsExt;
 
+use crate::extract::SeenFiles;
 use crate::filetype::*;
 use crate::seek_forward::SeekForward;
-use crate::SeenFiles;
 
-const CPIO_ALIGNMENT: u32 = 4;
-const CPIO_HEADER_LENGTH: u32 = 110;
+pub(crate) const CPIO_ALIGNMENT: u64 = 4;
+pub(crate) const CPIO_HEADER_LENGTH: u32 = 110;
 const CPIO_MAGIC_NUMBER: [u8; 6] = *b"070701";
 const PATH_MAX: usize = 4096;
+pub(crate) const TRAILER_FILENAME: &str = "TRAILER!!!";
+pub(crate) const TRAILER_SIZE: u64 = calculate_size(TRAILER_FILENAME, 0);
 
 #[derive(Debug, PartialEq)]
-pub struct Header {
-    pub ino: u32,
-    pub mode: u32,
-    pub uid: u32,
-    pub gid: u32,
-    pub nlink: u32,
-    pub mtime: u32,
-    pub filesize: u32,
+pub(crate) struct Header {
+    pub(crate) ino: u32,
+    pub(crate) mode: u32,
+    pub(crate) uid: u32,
+    pub(crate) gid: u32,
+    pub(crate) nlink: u32,
+    pub(crate) mtime: u32,
+    pub(crate) filesize: u32,
     major: u32,
     minor: u32,
-    pub rmajor: u32,
-    pub rminor: u32,
-    pub filename: String,
+    pub(crate) rmajor: u32,
+    pub(crate) rminor: u32,
+    pub(crate) filename: String,
 }
 
 impl Header {
     #![allow(clippy::too_many_arguments)]
-    pub fn new<S>(
+    pub(crate) fn new<S>(
         ino: u32,
         mode: u32,
         uid: u32,
@@ -63,7 +66,7 @@ impl Header {
         }
     }
 
-    pub fn trailer() -> Self {
+    pub(crate) fn trailer() -> Self {
         Self {
             ino: 0,
             mode: 0,
@@ -76,7 +79,7 @@ impl Header {
             minor: 0,
             rmajor: 0,
             rminor: 0,
-            filename: "TRAILER!!!".into(),
+            filename: TRAILER_FILENAME.into(),
         }
     }
 
@@ -85,16 +88,16 @@ impl Header {
         (u64::from(self.major) << 32) | u64::from(self.minor)
     }
 
-    pub fn is_root_directory(&self) -> bool {
+    pub(crate) fn is_root_directory(&self) -> bool {
         self.filename == "." && self.mode & MODE_FILETYPE_MASK == FILETYPE_DIRECTORY
     }
 
-    pub fn mode_perm(&self) -> u32 {
+    pub(crate) fn mode_perm(&self) -> u32 {
         self.mode & MODE_PERMISSION_MASK
     }
 
     // ls-style ASCII representation of the mode
-    pub fn mode_string(&self) -> [u8; 10] {
+    pub(crate) fn mode_string(&self) -> [u8; 10] {
         [
             match self.mode & MODE_FILETYPE_MASK {
                 FILETYPE_FIFO => b'p',
@@ -133,11 +136,11 @@ impl Header {
         ]
     }
 
-    fn padding_needed_for_file_content(&self) -> u32 {
+    fn padding_needed_for_file_content(&self) -> u64 {
         padding_needed_for(self.filesize.into(), CPIO_ALIGNMENT)
     }
 
-    pub fn permission(&self) -> Permissions {
+    pub(crate) fn permission(&self) -> Permissions {
         PermissionsExt::from_mode(self.mode & MODE_PERMISSION_MASK)
     }
 
@@ -145,16 +148,16 @@ impl Header {
         (u128::from(self.ino) << 64) | u128::from(self.dev())
     }
 
-    pub fn mark_seen(&self, seen_files: &mut SeenFiles) {
+    pub(crate) fn mark_seen(&self, seen_files: &mut SeenFiles) {
         seen_files.insert(self.ino_and_dev(), self.filename.clone());
     }
 
-    pub fn read<R: Read>(archive: &mut R) -> Result<Self> {
+    pub(crate) fn read<R: Read>(archive: &mut R) -> Result<Self> {
         let mut buffer = [0; CPIO_HEADER_LENGTH as usize];
         archive.read_exact(&mut buffer)?;
         check_begins_with_cpio_magic_header(&buffer)?;
         let namesize = hex_str_to_u32(&buffer[94..102])?;
-        let filename = read_filename(archive, namesize)?;
+        let filename = read_filename(archive, namesize.into())?;
         Ok(Self {
             ino: hex_str_to_u32(&buffer[6..14])?,
             mode: hex_str_to_u32(&buffer[14..22])?,
@@ -171,43 +174,47 @@ impl Header {
         })
     }
 
-    pub fn read_symlink_target<R: Read>(&self, archive: &mut R) -> Result<String> {
-        let align = self.padding_needed_for_file_content();
-        let mut target_bytes = vec![0u8; (self.filesize + align).try_into().unwrap()];
+    pub(crate) fn read_symlink_target<R: Read>(&self, archive: &mut R) -> Result<String> {
+        let align = usize::try_from(self.padding_needed_for_file_content()).unwrap();
+        let filesize = self.filesize.try_into().unwrap();
+        let mut target_bytes = vec![0u8; filesize + align];
         archive.read_exact(&mut target_bytes)?;
-        target_bytes.truncate(self.filesize.try_into().unwrap());
+        target_bytes.truncate(filesize);
         // TODO: propper name reading handling
         let target = std::str::from_utf8(&target_bytes).unwrap();
         Ok(target.into())
     }
 
-    pub fn skip_file_content<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
+    pub(crate) fn skip_file_content<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
         skip_file_content(archive, self.filesize)
     }
 
-    pub fn skip_file_content_padding<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
+    pub(crate) fn skip_file_content_padding<R: SeekForward>(&self, archive: &mut R) -> Result<()> {
         let skip = self.padding_needed_for_file_content();
         if skip == 0 {
             return Ok(());
         };
-        archive.seek_forward(skip.into())
+        archive.seek_forward(skip)
     }
 
-    pub fn try_get_hard_link_target<'a>(&self, seen_files: &'a SeenFiles) -> Option<&'a String> {
+    pub(crate) fn try_get_hard_link_target<'a>(
+        &self,
+        seen_files: &'a SeenFiles,
+    ) -> Option<&'a String> {
         if self.nlink <= 1 {
             return None;
         }
         seen_files.get(&self.ino_and_dev())
     }
 
-    pub fn write<W: Write>(&self, file: &mut W) -> Result<u64> {
+    pub(crate) fn write<W: Write>(&self, file: &mut W) -> Result<u64> {
         self.write_with_alignment(file, None, 0)
     }
 
-    pub fn write_with_alignment<W: Write>(
+    pub(crate) fn write_with_alignment<W: Write>(
         &self,
         file: &mut W,
-        alignment: Option<u32>,
+        alignment: Option<NonZeroU32>,
         written: u64,
     ) -> Result<u64> {
         // The filename needs to be terminated with \0.
@@ -220,8 +227,8 @@ impl Header {
         }
         let offset = u64::from(CPIO_HEADER_LENGTH) + u64::try_from(filename_len).unwrap();
         let mut padding_len;
-        if alignment.is_some_and(|alignment| self.filesize >= alignment) {
-            padding_len = padding_needed_for(written + offset, alignment.unwrap());
+        if alignment.is_some_and(|alignment| self.filesize >= alignment.into()) {
+            padding_len = padding_needed_for(written + offset, alignment.unwrap().get().into());
             let filename_plus_alignment = filename_len
                 .checked_add(padding_len.try_into().unwrap())
                 .unwrap();
@@ -245,20 +252,28 @@ impl Header {
             filename_len, self.filename,
             std::str::from_utf8(&padding).unwrap(),
         )?;
-        Ok(offset + u64::from(padding_len))
+        Ok(offset + padding_len)
     }
 
-    pub fn write_file_data_padding<W: Write>(&self, file: &mut W) -> Result<u64> {
+    pub(crate) fn write_file_data_padding<W: Write>(&self, file: &mut W) -> Result<u64> {
         let padding_len = self.padding_needed_for_file_content();
         if padding_len == 0 {
             return Ok(0);
         }
         let padding = vec![0u8; padding_len.try_into().unwrap()];
         file.write_all(&padding)?;
-        Ok(padding_len.into())
+        Ok(padding_len)
     }
 }
 
+/// Calculate the size of the header and file data plus the 4-byte padding
+pub(crate) const fn calculate_size(filename: &str, filesize: u64) -> u64 {
+    let filename_len = filename.len() as u64 + 1;
+    let mut size = CPIO_HEADER_LENGTH as u64 + filename_len;
+    size += padding_needed_for(size, CPIO_ALIGNMENT);
+    size + filesize + padding_needed_for(filesize, CPIO_ALIGNMENT)
+}
+
 fn check_begins_with_cpio_magic_header(header: &[u8]) -> std::io::Result<()> {
     if header[0..6] != CPIO_MAGIC_NUMBER {
         return Err(Error::new(
@@ -294,17 +309,16 @@ fn hex_str_to_u32(bytes: &[u8]) -> Result<u32> {
 
 /// Returns the amount of padding needed after `offset` to ensure that the
 /// following address will be aligned to `alignment`.
-fn padding_needed_for(offset: u64, alignment: u32) -> u32 {
-    // The rem operation is expected smaller than the right-hand side
-    let misalignment = (offset % u64::from(alignment)) as u32;
+pub(crate) const fn padding_needed_for(offset: u64, alignment: u64) -> u64 {
+    let misalignment = offset % alignment;
     if misalignment == 0 {
         return 0;
     }
     alignment - misalignment
 }
 
-fn read_filename<R: Read>(archive: &mut R, namesize: u32) -> Result<String> {
-    let header_align = padding_needed_for((CPIO_HEADER_LENGTH + namesize).into(), CPIO_ALIGNMENT);
+fn read_filename<R: Read>(archive: &mut R, namesize: u64) -> Result<String> {
+    let header_align = padding_needed_for(u64::from(CPIO_HEADER_LENGTH) + namesize, CPIO_ALIGNMENT);
     let mut filename_bytes = vec![0u8; (namesize + header_align).try_into().unwrap()];
     let filename_length: usize = (namesize - 1).try_into().unwrap();
     archive.read_exact(&mut filename_bytes)?;
@@ -327,25 +341,25 @@ fn read_filename<R: Read>(archive: &mut R, namesize: u32) -> Result<String> {
 ///
 /// Read the next cpio object header, check the magic, skip the file data.
 /// Return the file name.
-pub fn read_filename_from_next_cpio_object<R: Read + SeekForward>(
+pub(crate) fn read_file_name_and_size_from_next_cpio_object<R: Read + SeekForward>(
     archive: &mut R,
-) -> Result<String> {
+) -> Result<(String, u32)> {
     let mut header = [0; CPIO_HEADER_LENGTH as usize];
     archive.read_exact(&mut header)?;
     check_begins_with_cpio_magic_header(&header)?;
     let filesize = hex_str_to_u32(&header[54..62])?;
     let namesize = hex_str_to_u32(&header[94..102])?;
-    let filename = read_filename(archive, namesize)?;
+    let filename = read_filename(archive, namesize.into())?;
     skip_file_content(archive, filesize)?;
-    Ok(filename)
+    Ok((filename, filesize))
 }
 
 fn skip_file_content<R: SeekForward>(archive: &mut R, filesize: u32) -> Result<()> {
     if filesize == 0 {
         return Ok(());
     };
-    let skip = filesize + padding_needed_for(filesize.into(), CPIO_ALIGNMENT);
-    archive.seek_forward(skip.into())
+    let skip = u64::from(filesize) + padding_needed_for(filesize.into(), CPIO_ALIGNMENT);
+    archive.seek_forward(skip)
 }
 
 #[cfg(test)]
@@ -449,7 +463,7 @@ mod tests {
         let header = Header::new(42, 0o100_644, 0xAA, 0xBB, 1, 0x689CD1CC, 917184, 0, 0, path);
         let mut output = Vec::new();
         let size = header
-            .write_with_alignment(&mut output, Some(PATH_MAX as u32), 3956)
+            .write_with_alignment(&mut output, NonZeroU32::new(PATH_MAX as u32), 3956)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
@@ -467,7 +481,7 @@ mod tests {
         );
         let mut output = Vec::new();
         let size = header
-            .write_with_alignment(&mut output, Some(PATH_MAX as u32), 3988)
+            .write_with_alignment(&mut output, NonZeroU32::new(PATH_MAX as u32), 3988)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
