@@ -4,14 +4,16 @@
 use std::collections::HashMap;
 use std::fs::{symlink_metadata, Metadata};
 use std::io::{BufRead, BufWriter, Error, ErrorKind, Result, Write};
+use std::num::NonZeroU32;
 use std::os::unix::fs::{MetadataExt, PermissionsExt};
 
 use crate::compression::Compression;
 use crate::extended_error::ExtendedError;
 use crate::filetype::*;
-use crate::header::Header;
+use crate::header::{calculate_size, padding_needed_for, Header, TRAILER_SIZE};
 use crate::libc::{major, minor};
-use crate::{LOG_LEVEL_DEBUG, LOG_LEVEL_INFO};
+use crate::logger::Logger;
+use crate::CPIO_ALIGNMENT;
 
 #[derive(Debug, PartialEq)]
 struct Hardlink {
@@ -66,14 +68,14 @@ struct File {
 }
 
 #[derive(Debug, PartialEq)]
-pub struct Archive {
+pub(crate) struct Archive {
     compression: Compression,
     files: Vec<File>,
     hardlinks: HashMap<u128, Hardlink>,
 }
 
 #[derive(Debug, PartialEq)]
-pub struct Manifest {
+pub(crate) struct Manifest {
     archives: Vec<Archive>,
     umask: u32,
 }
@@ -553,30 +555,51 @@ impl Archive {
         self.compression = compression;
     }
 
-    fn write<W: Write>(
+    /// Calculate the size of the cpio archive (when using the standard 4-byte padding)
+    fn size(&self) -> u64 {
+        let mut size = 0;
+        for file in &self.files {
+            let filesize = match &file.filetype {
+                // Filesize of hardlinks are calculated later
+                Filetype::Hardlink { key: _, index: _ } => 0,
+                Filetype::Symlink { target } => u32::try_from(target.len()).unwrap(),
+                Filetype::EmptyFile
+                | Filetype::Directory
+                | Filetype::BlockDevice { major: _, minor: _ }
+                | Filetype::CharacterDevice { major: _, minor: _ }
+                | Filetype::Fifo
+                | Filetype::Socket => 0,
+            };
+            size += calculate_size(&file.name, filesize.into());
+        }
+        for hardlink in self.hardlinks.values() {
+            debug_assert!(hardlink.references > 0);
+            let filesize = hardlink.filesize.into();
+            size += filesize + padding_needed_for(filesize, CPIO_ALIGNMENT);
+        }
+        size + TRAILER_SIZE
+    }
+
+    fn write<W: Write, LW: Write>(
         &self,
         output_file: &mut W,
-        alignment: Option<u32>,
+        alignment: Option<NonZeroU32>,
         source_date_epoch: Option<u32>,
         mut size: u64,
-        log_level: u32,
+        logger: &mut Logger<LW>,
     ) -> Result<u64> {
         let mut next_ino = 0;
         let mut hardlink_ino = HashMap::new();
         let mut header;
         for file in &self.files {
-            if log_level >= LOG_LEVEL_INFO {
-                writeln!(std::io::stderr(), "{}", file.name)?;
-            }
+            info!(logger, "{}", file.name)?;
             (header, next_ino) = file.generate_header(next_ino, &self.hardlinks, &mut hardlink_ino);
             if let Some(epoch) = source_date_epoch {
                 if header.mtime > epoch {
                     header.mtime = epoch;
                 }
             }
-            if log_level >= LOG_LEVEL_DEBUG {
-                writeln!(std::io::stderr(), "{header:?}")?;
-            };
+            debug!(logger, "{header:?}")?;
             size += header.write_with_alignment(output_file, alignment, size)?;
             match &file.filetype {
                 Filetype::Hardlink { key, index: _ } => {
@@ -609,7 +632,10 @@ impl Manifest {
         Self { archives, umask }
     }
 
-    pub fn from_input<R: BufRead>(reader: R, log_level: u32) -> Result<Self> {
+    pub(crate) fn from_input<R: BufRead, W: Write>(
+        reader: R,
+        logger: &mut Logger<W>,
+    ) -> Result<Self> {
         let mut archives = vec![Archive::new()];
         let mut current_archive = archives.last_mut().unwrap();
         let mut umask = 0;
@@ -618,9 +644,7 @@ impl Manifest {
             let line = line.trim();
             if line.starts_with("#") || line.is_empty() {
                 if line.starts_with("#cpio") {
-                    if log_level >= LOG_LEVEL_DEBUG {
-                        eprintln!("Parsing line {}: {line}", line_number + 1);
-                    }
+                    debug!(logger, "Parsing line {}: {line}", line_number + 1)?;
                     if !current_archive.is_empty() {
                         archives.push(Archive::new());
                         current_archive = archives.last_mut().unwrap();
@@ -646,9 +670,7 @@ impl Manifest {
                 }
                 continue;
             }
-            if log_level >= LOG_LEVEL_DEBUG {
-                eprintln!("Parsing line {}: {line}", line_number + 1);
-            }
+            debug!(logger, "Parsing line {}: {line}", line_number + 1)?;
             let file_mask = current_archive
                 .add_line(line)
                 .map_err(|e| e.add_line(line_number + 1))?;
@@ -667,12 +689,12 @@ impl Manifest {
     }
 
     // Return the size in bytes of the uncompressed data.
-    pub fn write_archive(
+    pub(crate) fn write_archive<W: Write>(
         self,
         mut file: Option<std::fs::File>,
-        alignment: Option<u32>,
+        alignment: Option<NonZeroU32>,
         source_date_epoch: Option<u32>,
-        log_level: u32,
+        logger: &mut Logger<W>,
     ) -> Result<u64> {
         let mut size = 0;
         if let Some(file) = file.as_ref() {
@@ -682,29 +704,22 @@ impl Manifest {
             if archive.compression.is_uncompressed() {
                 if let Some(file) = file.as_mut() {
                     let mut writer = BufWriter::new(file);
-                    size = archive.write(
-                        &mut writer,
-                        alignment,
-                        source_date_epoch,
-                        size,
-                        log_level,
-                    )?;
+                    size =
+                        archive.write(&mut writer, alignment, source_date_epoch, size, logger)?;
                     writer.flush()?;
                 } else {
                     let mut stdout = std::io::stdout().lock();
-                    size = archive.write(
-                        &mut stdout,
-                        alignment,
-                        source_date_epoch,
-                        size,
-                        log_level,
-                    )?;
+                    size =
+                        archive.write(&mut stdout, alignment, source_date_epoch, size, logger)?;
                     stdout.flush()?;
                 }
             } else {
-                let mut compressor = archive.compression.compress(file, source_date_epoch)?;
+                let mut compressor =
+                    archive
+                        .compression
+                        .compress(file, source_date_epoch, || archive.size())?;
                 let mut writer = BufWriter::new(compressor.stdin.as_ref().unwrap());
-                size = archive.write(&mut writer, None, source_date_epoch, size, log_level)?;
+                size = archive.write(&mut writer, None, source_date_epoch, size, logger)?;
                 writer.flush()?;
                 drop(writer);
                 let exit_status = compressor.wait()?;
@@ -742,9 +757,9 @@ mod tests {
     use std::path::Path;
 
     use super::*;
+    use crate::logger::Level;
     use crate::temp_dir::TempDir;
     use crate::tests::TEST_LOCK;
-    use crate::LOG_LEVEL_WARNING;
 
     fn create_text_file_in_tmpdir<P: AsRef<Path>>(
         tempdir: &TempDir,
@@ -757,7 +772,7 @@ mod tests {
         path.into_os_string().into_string().unwrap()
     }
 
-    pub fn make_temp_dir_with_hardlinks() -> Result<TempDir> {
+    pub(crate) fn make_temp_dir_with_hardlinks() -> Result<TempDir> {
         let temp_dir = TempDir::new()?;
         let path = temp_dir.path.join("a");
         let mut file = std::fs::File::create(&path)?;
@@ -1268,7 +1283,8 @@ mod tests {
         # This is a comment\n\n\
         /bin\tbin\tdir\t755\t0\t0\t1681992796\n\
         /usr/bin/gzip\tbin/gzip\tfile\t755\t0\t0\t1739259005\t35288\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let stat = symlink_metadata("/usr/bin/gzip").unwrap();
         let key = get_hardlink_key(&stat);
         let expected_archive = Archive::with_files_and_hardlinks(
@@ -1286,6 +1302,11 @@ mod tests {
             HashMap::from([(key, Hardlink::new("/usr/bin/gzip", 35288))]),
         );
         assert_eq!(manifest, Manifest::new(vec![expected_archive], 0o022));
+        assert_eq!(
+            logger.get_logs(),
+            "Parsing line 3: /bin\tbin\tdir\t755\t0\t0\t1681992796\n\
+            Parsing line 4: /usr/bin/gzip\tbin/gzip\tfile\t755\t0\t0\t1739259005\t35288\n",
+        );
     }
 
     #[test]
@@ -1293,7 +1314,8 @@ mod tests {
         let input = b"\
         #cpio: zstd -1\n\
         /bin\tbin\tdir\t755\t0\t0\t1681992796\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let expected_archive = Archive::with_files_compressed(
             vec![File::new(
                 Filetype::Directory,
@@ -1306,6 +1328,7 @@ mod tests {
             Compression::Zstd { level: Some(1) },
         );
         assert_eq!(manifest, Manifest::new(vec![expected_archive], 0));
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1316,7 +1339,8 @@ mod tests {
         /bin\tbin\tdir\t755\t0\t0\t1681992796\n\
         #cpio\n\
         /\t.\tdir\t755\t0\t0\t1732230747\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let expected_manifest = Manifest::new(
             vec![
                 Archive::with_files(vec![File::new(
@@ -1339,39 +1363,52 @@ mod tests {
             0,
         );
         assert_eq!(manifest, expected_manifest);
+        assert_eq!(
+            logger.get_logs(),
+            "Parsing line 3: #cpio\n\
+            Parsing line 4: /bin\tbin\tdir\t755\t0\t0\t1681992796\n\
+            Parsing line 5: #cpio\n\
+            Parsing line 6: /\t.\tdir\t755\t0\t0\t1732230747\n",
+        );
     }
 
     #[test]
     fn test_manifest_from_input_file_not_found() {
         let input = b"/nonexistent\n";
-        let got = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap_err();
+        let mut logger = Logger::new_vec(Level::Info);
+        let got = Manifest::from_input(input.as_ref(), &mut logger).unwrap_err();
         assert_eq!(got.kind(), ErrorKind::NotFound);
         assert_eq!(
             got.to_string(),
             "line 1: /nonexistent: No such file or directory (os error 2)"
         );
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
     fn test_manifest_from_input_invalid_cpio_directive() {
         let input = b" #cpio \n #cpio:  zstd  \n #cpio something -42  ";
-        let got = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap_err();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let got = Manifest::from_input(input.as_ref(), &mut logger).unwrap_err();
         assert_eq!(got.kind(), ErrorKind::InvalidInput);
         assert_eq!(
             got.to_string(),
             "line 3: Unknown cpio archive directive: #cpio something -42"
         );
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
     fn test_manifest_from_input_unknown_compressor() {
         let input = b"#cpio: brotli\n";
-        let got = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap_err();
+        let mut logger = Logger::new_vec(Level::Info);
+        let got = Manifest::from_input(input.as_ref(), &mut logger).unwrap_err();
         assert_eq!(got.kind(), ErrorKind::InvalidData);
         assert_eq!(
             got.to_string(),
             "line 1: Unknown compression format: brotli"
         );
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1379,10 +1416,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: bzip2 -3\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Info);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1396,6 +1434,7 @@ mod tests {
             \xe5l\xc6e\xf5\xba\xaf\x8b\xb9\"\x9c(H\x09\x1eOY\x80"
         );
         assert_eq!(read, 66);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1403,10 +1442,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: gzip -7\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1419,6 +1459,7 @@ mod tests {
             A\x8a\x8a\x8a\x0c@\0\0N\xe5\x097|\0\0\0"
         );
         assert_eq!(read, 48);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1426,10 +1467,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: lz4 -4\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1442,6 +1484,7 @@ mod tests {
             AILER!!!\0\0\0\0"
         );
         assert_eq!(read, 44);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1449,10 +1492,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: lzma -1\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1465,6 +1509,7 @@ mod tests {
             pL\xa1]\xb0mv\xe7&\xc4o\xff\xfe$\x90\0"
         );
         assert_eq!(read, 48);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1472,9 +1517,10 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: lzop -9\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
-        let got = manifest.write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING);
+        let got = manifest.write_archive(Some(file), None, Some(1754439117), &mut logger);
         if got
             .as_ref()
             .is_err_and(|e| e.to_string() == "Program 'lzop' not found in PATH.")
@@ -1502,6 +1548,7 @@ mod tests {
             !\0@\0\x11\0\0\0\0\0\0"
         );
         assert_eq!(read, 91);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1509,10 +1556,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: xz -6\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1527,6 +1575,7 @@ mod tests {
             \0\x014|\xcb{\x1f\xc2\x90B\x99\x0d\x01\0\0\0\0\x01YZ"
         );
         assert_eq!(read, 84);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1534,10 +1583,11 @@ mod tests {
         let temp_dir = TempDir::new().unwrap();
         let path = temp_dir.path.join("initrd.img");
         let input = b"#cpio: zstd -2\n";
-        let manifest = Manifest::from_input(input.as_ref(), LOG_LEVEL_WARNING).unwrap();
+        let mut logger = Logger::new_vec(Level::Warning);
+        let manifest = Manifest::from_input(input.as_ref(), &mut logger).unwrap();
         let file = std::fs::File::create(&path).unwrap();
         let size = manifest
-            .write_archive(Some(file), None, Some(1754439117), LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, Some(1754439117), &mut logger)
             .unwrap();
         assert_eq!(size, 124);
         let mut written_file = std::fs::File::open(&path).unwrap();
@@ -1545,11 +1595,12 @@ mod tests {
         let read = written_file.read_to_end(&mut output).unwrap();
         assert_eq!(
             output,
-            b"(\xb5/\xfd\x04P\x15\x01\0\xc8070701\
+            b"(\xb5/\xfd$|\x15\x01\0\xc8070701\
             010B0TRAILER!!!\0\
             \0\0\0\x03\x10\0\x19\xde\x89?F\x95\xfb\x16m"
         );
         assert_eq!(read, 47);
+        assert_eq!(logger.get_logs(), "");
     }
 
     #[test]
@@ -1559,13 +1610,20 @@ mod tests {
         let archive = Archive::with_files_compressed(vec![root_dir], Compression::Failing);
         let manifest = Manifest::new(vec![archive], 0o022);
         let file = std::fs::File::create(temp_dir.path.join("initrd.img")).unwrap();
+        let mut logger = Logger::new_vec(Level::Debug);
         let got = manifest
-            .write_archive(Some(file), None, None, LOG_LEVEL_WARNING)
+            .write_archive(Some(file), None, None, &mut logger)
             .unwrap_err();
         assert!(
             matches!(got.kind(), ErrorKind::Other if got.to_string() == "false failed: exit status: 1")
                 || matches!(got.kind(), ErrorKind::BrokenPipe)
         );
+        assert_eq!(
+            logger.get_logs(),
+            ".\n\
+            Header { ino: 0, mode: 16877, uid: 819, gid: 66, nlink: 2, mtime: 1749125499, \
+            filesize: 0, major: 0, minor: 0, rmajor: 0, rminor: 0, filename: \".\" }\n",
+        );
     }
 
     #[test]
@@ -1609,8 +1667,9 @@ mod tests {
             File::new(Filetype::EmptyFile, "fstab", 0o644, 0x339, 0x48, 0x6E44C280),
         ]);
         let mut output = Vec::new();
+        let mut logger = Logger::new_vec(Level::Info);
         let size = archive
-            .write(&mut output, None, Some(0x6B49D200), 0, LOG_LEVEL_WARNING)
+            .write(&mut output, None, Some(0x6B49D200), 0, &mut logger)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
@@ -1640,6 +1699,11 @@ mod tests {
             TRAILER!!!\0\0\0\0",
         );
         assert_eq!(size, 952);
+        assert_eq!(archive.size(), 952);
+        assert_eq!(
+            logger.get_logs(),
+            ".\nloop0\nconsole\nsbin\ninitctl\nnotify\nfstab\n"
+        );
     }
 
     #[test]
@@ -1677,13 +1741,14 @@ mod tests {
             hardlinks,
         );
         let mut output = Vec::new();
+        let mut logger = Logger::new_vec(Level::Info);
         let size = archive
             .write(
                 &mut output,
-                Some(32),
+                NonZeroU32::new(32),
                 Some(0x6B49D200),
                 0,
-                LOG_LEVEL_WARNING,
+                &mut logger,
             )
             .unwrap();
         assert_eq!(
@@ -1705,6 +1770,8 @@ mod tests {
             TRAILER!!!\0\0\0\0",
         );
         assert_eq!(size, 560);
+        assert_eq!(archive.size(), 536);
+        assert_eq!(logger.get_logs(), ".\nexample\nsmall.txt\n");
     }
 
     #[test]
@@ -1740,8 +1807,9 @@ mod tests {
             HashMap::from([(8921120, Hardlink::with_references(&path, 7, 2))]),
         );
         let mut output = Vec::new();
+        let mut logger = Logger::new_vec(Level::Info);
         let size = archive
-            .write(&mut output, None, None, 0, LOG_LEVEL_WARNING)
+            .write(&mut output, None, None, 0, &mut logger)
             .unwrap();
         assert_eq!(
             std::str::from_utf8(&output).unwrap(),
@@ -1756,5 +1824,7 @@ mod tests {
             TRAILER!!!\0\0\0\0",
         );
         assert_eq!(size, 356);
+        assert_eq!(archive.size(), 356);
+        assert_eq!(logger.get_logs(), "a\nb\n");
     }
 }
