@@ -3,8 +3,9 @@
 
 use std::env;
 use std::error::Error;
-use std::fs::File;
+use std::fs::{symlink_metadata, File};
 use std::io::{ErrorKind, Read, Write};
+use std::os::unix::fs::MetadataExt;
 use std::process::{Command, Output, Stdio};
 use std::time::SystemTime;
 
@@ -145,7 +146,7 @@ fn test_create_compressed_cpio_on_stdout() -> Result<(), Box<dyn Error>> {
         .assert_stderr("Compression level 42 higher than maximum, reducing to 19.\n");
     assert_eq!(
         output.stdout,
-        b"(\xb5/\xfd\x04h\x0d\x02\0\x02\xc3\x0a\x11\x90M\x07\
+        b"(\xb5/\xfd$\xf0\x0d\x02\0\x02\xc3\x0a\x11\x90M\x07\
         \xa0\xff\x18S\x04G\xf3[\xc9\xb1\xef\x8eT\x06m\x0b\
         \0h\x8a-\xd3\xdc\xe7l\xfb`\\\x8c\x06\x0a)\x04\
         \x09'\x95\xe2\xbc\\\x0e\x08 \xc0s\x07\x19\xde\x89v\
@@ -266,7 +267,7 @@ fn test_create_data_align_zero() -> Result<(), Box<dyn Error>> {
 
     cmd.output()?
         .assert_failure(2)
-        .assert_stderr_contains("Error: --data-align must be a multiple of 4 bytes")
+        .assert_stderr_contains("Error: --data-align must be a positive number")
         .assert_stdout("");
     Ok(())
 }
@@ -287,7 +288,7 @@ fn test_create_uncompressed_plus_zstd_on_stdout() -> Result<(), Box<dyn Error>>
         070701000000000000000000000000000000000000000100000000\
         00000000000000000000000000000000000000000000000B00000000\
         TRAILER!!!\0\0\0\0\
-        (\xb5/\xfd\x04P\x15\x01\0\xc8070701\
+        (\xb5/\xfd$|\x15\x01\0\xc8070701\
         010B0TRAILER!!!\0\
         \0\0\0\x03\x10\0\x19\xde\x89?F\x95\xfb\x16m",
     );
@@ -316,18 +317,23 @@ fn test_count_unexpected_argument() -> Result<(), Box<dyn Error>> {
 }
 
 #[test]
-fn test_examine_compressed_cpio() -> Result<(), Box<dyn Error>> {
+fn test_examine_compressed_cpio_raw() -> Result<(), Box<dyn Error>> {
     for compression in ["bzip2", "gzip", "lz4", "lzop", "xz", "zstd"] {
         if program_not_available(compression) {
             continue;
         }
+        let path = format!("tests/{compression}.cpio");
         let mut cmd = get_command();
-        cmd.arg("-e").arg(format!("tests/{compression}.cpio"));
+        cmd.arg("-e").arg(&path).arg("--raw");
+        let size = symlink_metadata(&path)?.size();
 
         cmd.output()?
             .assert_stderr("")
             .assert_success()
-            .assert_stdout(format!("0\tcpio\n512\t{compression}\n"));
+            .assert_stdout(format!(
+                "0\t512\t512\tcpio\t8\n512\t{size}\t{}\t{compression}\t56\n",
+                size - 512
+            ));
     }
     Ok(())
 }
@@ -369,11 +375,14 @@ fn test_extract_make_directories_with_pattern() -> Result<(), Box<dyn Error>> {
 }
 
 #[test]
-fn test_examine_single_cpio() -> Result<(), Box<dyn Error>> {
+fn test_examine_single_cpio_raw() -> Result<(), Box<dyn Error>> {
     let mut cmd = get_command();
-    cmd.arg("-e").arg("tests/single.cpio");
+    cmd.arg("-e").arg("--raw").arg("tests/single.cpio");
 
-    cmd.output()?.assert_success().assert_stdout("0\tcpio\n");
+    cmd.output()?
+        .assert_stderr("")
+        .assert_success()
+        .assert_stdout("0\t512\t512\tcpio\t8\n");
     Ok(())
 }
 
